#!/usr/bin/env python
import logging
from datetime import datetime
from pathlib import Path
from zoneinfo import ZoneInfo

from django.conf import locale
from edc_constants.internationalization import EXTRA_LANG_INFO
from edc_test_utils import DefaultTestSettings, func_main
from multisite import SiteID

LANG_INFO = dict(locale.LANG_INFO, **EXTRA_LANG_INFO)
locale.LANG_INFO = LANG_INFO
LANGUAGE_LIST = ["sw", "en-gb", "en", "mas", "ry", "lg", "rny"]

app_name = "intecomm_edc"
base_dir = Path(__file__).absolute().parent

project_settings = DefaultTestSettings(
    calling_file=__file__,
    EDC_EGFR_DROP_NOTIFICATION_MODEL="intecomm_subject.egfrdropnotification",
    EDC_RANDOMIZATION_REGISTER_DEFAULT_RANDOMIZER=False,
    ROOT_URLCONF="intecomm_edc.urls",
    EDC_AUTH_CODENAMES_WARN_ONLY=True,
    EDC_DX_REVIEW_LIST_MODEL_APP_LABEL="edc_dx_review",
    EDC_DX_REVIEW_APP_LABEL="intecomm_subject",
    EDC_APPOINTMENT_ALLOW_SKIPPED_APPT_USING={
        "intecomm_subject.nextappointment": ("appt_date", "visitschedule")
    },
    BASE_DIR=base_dir,
    APP_NAME=app_name,
    SITE_ID=SiteID(default=101),
    SENTRY_ENABLED=False,
    INDEX_PAGE="localhost:8000",
    LANGUAGE_CODE="en",
    LANGUAGES=[(code, LANG_INFO[code]["name"]) for code in LANGUAGE_LIST],
    EXPORT_FOLDER=str(base_dir / "tests" / "export"),
    SUBJECT_APP_LABEL="intecomm_subject",
    SUBJECT_SCREENING_MODEL="intecomm_screening.subjectscreening",
    SUBJECT_VISIT_MODEL="intecomm_subject.subjectvisit",
    SUBJECT_VISIT_MISSED_MODEL="intecomm_subject.subjectvisitmissed",
    SUBJECT_CONSENT_MODEL="intecomm_consent.subjectconsent",
    SUBJECT_REQUISITION_MODEL="intecomm_subject.subjectrequisition",
    SUBJECT_LOCATOR_MODEL="intecomm_prn.subjectlocator",
    EDC_HE_ASSETS_MODEL="intecomm_subject.healtheconomicsassets",
    EDC_HE_HOUSEHOLDHEAD_MODEL="intecomm_subject.healtheconomicshouseholdhead",
    EDC_HE_INCOME_MODEL="intecomm_subject.healtheconomicsincome",
    EDC_HE_PATIENT_MODEL="intecomm_subject.healtheconomicspatient",
    EDC_HE_PROPERTY_MODEL="intecomm_subject.healtheconomicsproperty",
    EDC_BLOOD_RESULTS_MODEL_APP_LABEL="intecomm_subject",
    DEFENDER_ENABLED=False,
    DJANGO_LAB_DASHBOARD_REQUISITION_MODEL="intecomm_subject.subjectrequisition",
    ADVERSE_EVENT_ADMIN_SITE="intecomm_ae_admin",
    EDC_DX_LABELS=dict(hiv="HIV", dm="Diabetes", htn="Hypertension"),
    ADVERSE_EVENT_APP_LABEL="intecomm_ae",
    EDC_NAVBAR_DEFAULT="intecomm_dashboard",
    EDC_PROTOCOL_STUDY_OPEN_DATETIME=datetime(2019, 4, 30, 0, 0, 0, tzinfo=ZoneInfo("UTC")),
    EDC_PROTOCOL_STUDY_CLOSE_DATETIME=datetime(
        2025, 12, 31, 23, 59, 59, tzinfo=ZoneInfo("UTC")
    ),
    DJANGO_LANGUAGES=dict(
        en="English",
        lg="Luganda",
        rny="Runyankore",
        ry="Runyakitara",
        mas="Maasai",
    ),
    DASHBOARD_BASE_TEMPLATES=dict(
        edc_base_template="edc_dashboard/base.html",
        listboard_base_template="intecomm_edc/base.html",
        dashboard_base_template="intecomm_edc/base.html",
        screening_listboard_template="intecomm_dashboard/screening/listboard.html",
        subject_listboard_template=(
            "intecomm_dashboard/bootstrap3/subject/subject_listboard.html"
        ),
        subject_dashboard_template="intecomm_dashboard/subject/dashboard.html",
        subject_review_listboard_template="edc_review_dashboard/subject_review_listboard.html",
    ),
    ETC_DIR=str(base_dir / "intecomm_edc" / "tests" / "etc"),
    EDC_BOOTSTRAP=3,
    EMAIL_BACKEND="django.core.mail.backends.locmem.EmailBackend",
    EMAIL_CONTACTS={
        "data_request": "someone@example.com",
        "data_manager": "someone@example.com",
        "tmg": "someone@example.com",
        "ae_reports": "aereports@example.com",
    },
    EMAIL_ENABLED=True,
    HOLIDAY_FILE=str(base_dir / "intecomm_edc" / "tests" / "holidays.csv"),
    LIVE_SYSTEM=False,
    EDC_RANDOMIZATION_LIST_PATH=str(base_dir / "intecomm_edc" / "tests" / "etc"),
    EDC_SITES_MODULE_NAME="intecomm_sites",
    EDC_AUTH_SKIP_SITE_AUTHS=True,
    EDC_AUTH_SKIP_AUTH_UPDATER=True,
    INSTALLED_APPS=[
        "django.contrib.admin",
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sessions",
        "django.contrib.messages",
        "django.contrib.staticfiles",
        "django.contrib.sites",
        "django_crypto_fields.apps.AppConfig",
        "django_revision.apps.AppConfig",
        # "debug_toolbar",
        "django_extensions",
        "logentry_admin",
        "multisite",
        "simple_history",
        "storages",
        "edc_sites.apps.AppConfig",
        "edc_action_item.apps.AppConfig",
        "edc_appointment.apps.AppConfig",
        "edc_auth.apps.AppConfig",
        "edc_adverse_event.apps.AppConfig",
        "edc_consent.apps.AppConfig",
        "edc_crf.apps.AppConfig",
        "edc_reportable.apps.AppConfig",
        "edc_lab.apps.AppConfig",
        "edc_visit_schedule.apps.AppConfig",
        "edc_visit_tracking.apps.AppConfig",
        "edc_dx.apps.AppConfig",
        "edc_dx_review.apps.AppConfig",
        "edc_device.apps.AppConfig",
        "edc_dashboard.apps.AppConfig",
        "edc_data_manager.apps.AppConfig",
        "edc_export.apps.AppConfig",
        "edc_facility.apps.AppConfig",
        "edc_fieldsets.apps.AppConfig",
        "edc_form_validators.apps.AppConfig",
        "edc_form_runners.apps.AppConfig",
        "edc_he.apps.AppConfig",
        "edc_lab_dashboard.apps.AppConfig",
        "edc_label.apps.AppConfig",
        "edc_list_data.apps.AppConfig",
        "edc_listboard.apps.AppConfig",
        "edc_identifier.apps.AppConfig",
        "edc_locator.apps.AppConfig",
        "edc_metadata.apps.AppConfig",
        "edc_model.apps.AppConfig",
        "edc_model_fields.apps.AppConfig",
        "edc_model_admin.apps.AppConfig",
        "edc_navbar.apps.AppConfig",
        "edc_next_appointment.apps.AppConfig",  # keep for migrate
        "edc_notification.apps.AppConfig",
        "edc_offstudy.apps.AppConfig",
        "edc_pharmacy.apps.AppConfig",
        "edc_pdutils.apps.AppConfig",
        "edc_protocol.apps.AppConfig",
        "edc_protocol_incident.apps.AppConfig",
        "edc_prn.apps.AppConfig",
        "edc_qol.apps.AppConfig",
        "edc_randomization.apps.AppConfig",
        "edc_refusal.apps.AppConfig",
        "edc_registration.apps.AppConfig",
        "edc_pdf_reports.apps.AppConfig",
        "edc_review_dashboard.apps.AppConfig",
        "edc_rx.apps.AppConfig",
        "edc_screening.apps.AppConfig",
        "edc_subject_dashboard.apps.AppConfig",
        "edc_timepoint.apps.AppConfig",
        "edc_unblinding.apps.AppConfig",
        "edc_form_describer.apps.AppConfig",
        "edc_adherence.apps.AppConfig",
        "intecomm_rando.apps.AppConfig",
        "intecomm_consent.apps.AppConfig",
        "intecomm_lists.apps.AppConfig",
        "intecomm_dashboard.apps.AppConfig",
        "intecomm_facility.apps.AppConfig",
        "intecomm_labs.apps.AppConfig",
        "intecomm_subject.apps.AppConfig",
        "intecomm_visit_schedule.apps.AppConfig",
        "intecomm_ae.apps.AppConfig",
        "intecomm_auth.apps.AppConfig",
        "intecomm_prn.apps.AppConfig",
        "intecomm_export.apps.AppConfig",
        "intecomm_group.apps.AppConfig",
        "intecomm_screening.apps.AppConfig",
        "intecomm_sites.apps.AppConfig",
        "intecomm_edc.apps.AppConfig",
        "edc_appconfig.apps.AppConfig",
    ],
    add_dashboard_middleware=True,
    add_lab_dashboard_middleware=True,
    add_adverse_event_dashboard_middleware=True,
).settings


def main():
    tests = [
        "intecomm_auth.tests",
        "intecomm_ae.tests",
        "intecomm_dashboard.tests",
        "intecomm_edc.tests",
        "intecomm_labs.tests",
        "intecomm_lists.tests",
        "intecomm_prn.tests",
        "intecomm_group.tests",
        "intecomm_screening.tests",
        "intecomm_subject.tests",
        "intecomm_consent.tests",
        "intecomm_visit_schedule.tests",
        "intecomm_form_validators.tests",
        "intecomm_rando.tests",
        "intecomm_eligibility.tests",
    ]

    func_main(project_settings, *tests)


if __name__ == "__main__":
    logging.basicConfig()
    main()



from django.contrib import admin
from django.db.models import Q
from edc_he.modeladmin_mixins import HealthEconomicsHouseholdHeadModelAdminMixin
from edc_he.models import Ethnicities, Religions

from ...admin_site import intecomm_subject_admin
from ...forms import HealthEconomicsHouseholdHeadForm
from ...models import HealthEconomicsHouseholdHead
from ..modeladmin_mixins import CrfModelAdmin


@admin.register(HealthEconomicsHouseholdHead, site=intecomm_subject_admin)
class HealthEconomicsHouseholdHeadAdmin(
    HealthEconomicsHouseholdHeadModelAdminMixin, CrfModelAdmin
):
    form = HealthEconomicsHouseholdHeadForm

    save_on_top = True

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if getattr(request, "site", None):
            if db_field.name == "hoh_ethnicity":
                kwargs["queryset"] = Ethnicities.objects.filter(
                    Q(extra_value=request.site.siteprofile.country)
                    | Q(extra_value__isnull=True)
                )
            if db_field.name == "hoh_religion":
                kwargs["queryset"] = Religions.objects.filter(
                    Q(extra_value=request.site.siteprofile.country)
                    | Q(extra_value__isnull=True)
                )
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        if getattr(request, "site", None):
            if db_field.name == "hoh_insurance":
                model_cls = getattr(self.model, db_field.name).field.related_model
                kwargs["queryset"] = model_cls.objects.filter(
                    Q(extra_value=request.site.siteprofile.country)
                    | Q(extra_value__isnull=True)
                )
        return super().formfield_for_manytomany(db_field, request, **kwargs)



from edc_visit_schedule.schedule import Schedule

from intecomm_consent.consents import cdef_tz_v1, cdef_ug_v1

from ..constants import FACILITY_SCHEDULE
from .visits import get_followup_visit, visit00, visit01, visit12

facility_schedule = Schedule(
    name=FACILITY_SCHEDULE,
    verbose_name="Facility-based integrated care",
    onschedule_model="intecomm_prn.onscheduleinte",
    offschedule_model="intecomm_prn.offscheduleinte",
    consent_definitions=[cdef_tz_v1, cdef_ug_v1],
    appointment_model="edc_appointment.appointment",
)

visits = [visit00, visit01]
for month in range(2, 12):
    visits.append(get_followup_visit(month, start=1, allow_unscheduled=True))
visits.append(visit12)

for visit in visits:
    facility_schedule.add_visit(visit=visit)



# Generated by Django 4.2.7 on 2023-12-04 22:19

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_consent", "0016_alter_historicalsubjectconsent_language_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="subjectconsent",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Subject Consent",
                "verbose_name_plural": "Subject Consents",
            },
        ),
        migrations.AlterModelOptions(
            name="subjectreconsent",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Subject re-consent",
            },
        ),
        migrations.AlterUniqueTogether(
            name="subjectconsent",
            unique_together=set(),
        ),
        migrations.AddField(
            model_name="historicalsubjectconsent",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectconsent",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectconsentug",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectconsentug",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectreconsent",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectreconsent",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="subjectconsent",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="subjectconsent",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="subjectreconsent",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="subjectreconsent",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
    ]



from django import forms
from edc_action_item.forms import ActionItemCrfFormMixin
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_lab_panel.panels import lft_panel
from edc_lab_results.form_validator_mixins import BloodResultsFormValidatorMixin

from ...models import BloodResultsLft


class BloodResultsLftFormValidator(BloodResultsFormValidatorMixin, CrfFormValidator):
    panel = lft_panel


class BloodResultsLftForm(ActionItemCrfFormMixin, CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = BloodResultsLftFormValidator

    class Meta(ActionItemCrfFormMixin.Meta):
        model = BloodResultsLft
        fields = "__all__"



from django import forms
from edc_form_validators import FormValidatorMixin
from edc_locator.forms import SubjectLocatorFormValidator
from edc_model_form.mixins import BaseModelFormMixin
from edc_offstudy.modelform_mixins import OffstudyNonCrfModelFormMixin
from edc_sites.modelform_mixins import SiteModelFormMixin

__all__ = ["SubjectLocatorForm"]


class SubjectLocatorForm(
    SiteModelFormMixin,
    OffstudyNonCrfModelFormMixin,
    BaseModelFormMixin,
    FormValidatorMixin,
    forms.ModelForm,
):
    form_validator_cls = SubjectLocatorFormValidator

    subject_identifier = forms.CharField(
        label="Subject Identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )



from .consent_refusal_form import ConsentRefusalForm
from .health_talk_log_form import HealthTalkLogForm
from .patient_call_form import PatientCallForm
from .patient_group_form import PatientGroupForm
from .patient_log import PatientLogForm, PatientLogUgForm
from .subject_screening import SubjectScreeningForm, SubjectScreeningUgForm



# Generated by Django 4.1.7 on 2023-04-26 21:04

import _socket
from django.db import migrations, models
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0020_consentrefusal_historicalconsentrefusal_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="IdenfifierFormat",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("format_name", models.CharField(max_length=50)),
                ("format_regex", models.CharField(max_length=150)),
                ("comment", models.TextField(blank=True, null=True)),
                (
                    "health_facility",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="intecomm_screening.healthfacility",
                    ),
                ),
            ],
            options={
                "verbose_name": "Idenfifier format",
                "verbose_name_plural": "Idenfifier formats",
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
            },
        ),
    ]



from __future__ import annotations

from django.contrib import admin

from ..admin_site import intecomm_prn_admin
from ..forms import OnScheduleInteForm
from ..models import OnScheduleInte
from .modeladmin_mixins import OnScheduleModelAdmin


@admin.register(OnScheduleInte, site=intecomm_prn_admin)
class OnScheduleInteAdmin(OnScheduleModelAdmin):
    form = OnScheduleInteForm



from django import forms
from django.utils.translation import gettext as _
from edc_constants.constants import BOTH, NO, OTHER, OUTPATIENT, YES
from edc_crf.modelform_mixins import CrfSingletonModelFormMixin
from edc_form_validators import INVALID_ERROR, FormValidator

from ...constants import ALONE, PAID_WORK
from ...models import CareseekingB
from ..mixins import CrfModelFormMixin


class CareseekingBFormValidator(FormValidator):
    def clean(self) -> None:
        self.applicable_if(YES, field="ill_month", field_applicable="seek_advice")
        self.applicable_if(NO, field="seek_advice", field_applicable="no_seek_advice")
        self.validate_other_specify(
            field="no_seek_advice", other_specify_field="no_seek_advice_other"
        )
        self.applicable_if(YES, field="seek_advice", field_applicable="seek_facility")
        self.validate_other_specify(
            field="seek_advice", other_specify_field="seek_advice_other"
        )
        self.applicable_if(YES, field="seek_advice", field_applicable="seek_care_type")
        self.required_if(
            OUTPATIENT,
            BOTH,
            field="seek_care_type",
            field_required="outpatient_visits",
            field_required_evaluate_as_int=True,
        )

        self.m2m_required_if(YES, field="med_prescribed", m2m_field="med_conditions")
        self.m2m_other_specify(m2m_field="med_conditions", field_other="med_conditions_other")
        self.applicable_if(YES, field="med_prescribed", field_applicable="med_collected")
        self.applicable_if(
            NO, field="med_collected", field_applicable="med_not_collected_reason"
        )
        self.validate_other_specify(
            field="med_not_collected_reason",
            other_specify_field="med_not_collected_reason_other",
        )
        self.required_if(
            YES,
            field="med_collected",
            field_required="med_cost_tot",
            field_required_evaluate_as_int=True,
        )

        self.applicable_if(YES, field="tests_requested", field_applicable="tests_done")
        self.required_if(NO, field="tests_done", field_required="tests_not_done_reason")
        self.validate_other_specify(
            field="tests_not_done_reason", other_specify_field="tests_not_done_other"
        )
        self.required_if(
            YES,
            field="tests_done",
            field_required="tests_cost",
            field_required_evaluate_as_int=True,
        )

        self.validate_other_specify(
            field="missed_activities", other_specify_field="missed_activities_other"
        )
        self.required_if(
            PAID_WORK,
            OTHER,
            field="missed_activities",
            field_required="care_visit_lost_income",
            field_required_evaluate_as_int=True,
        )

        self.m2m_single_selection_if(ALONE, m2m_field="accompany")
        if (
            self.cleaned_data.get("accompany_num")
            and ALONE in [o.name for o in self.cleaned_data.get("accompany")]
            and self.cleaned_data.get("accompany_num") != 0
        ):
            self.raise_validation_error(
                {"accompany_num": _("Expected 0 based on response above")}, INVALID_ERROR
            )
        elif (
            self.cleaned_data.get("accompany_num")
            and ALONE not in [o.name for o in self.cleaned_data.get("accompany") or []]
            and self.cleaned_data.get("accompany_num") == 0
        ):
            self.raise_validation_error(
                {
                    "accompany_num": _(
                        "Expected a value greater than 0 based on response above"
                    )
                },
                INVALID_ERROR,
            )

        self.applicable_if_true(
            ALONE not in [o.name for o in self.cleaned_data.get("accompany") or []],
            field_applicable="accompany_wait",
        )
        self.applicable_if_true(
            ALONE not in [o.name for o in self.cleaned_data.get("accompany") or []],
            field_applicable="accompany_alt",
        )
        self.validate_other_specify(
            field="accompany_alt", other_specify_field="accompany_alt_other"
        )
        self.required_if(
            PAID_WORK,
            OTHER,
            field="accompany_alt",
            field_required="accompany_lost_income",
            field_required_evaluate_as_int=True,
        )

        self.m2m_other_specify(m2m_field="money_sources", field_other="money_sources_other")

        if self.cleaned_data.get("money_source_main") and self.cleaned_data.get(
            "money_source_main"
        ) not in [o.name for o in self.cleaned_data.get("money_sources")]:
            self.raise_validation_error(
                {"money_source_main": _("Response not found among responses given above")},
                INVALID_ERROR,
            )
        self.validate_other_specify(
            field="money_source_main", other_specify_field="money_source_main_other"
        )

        self.required_if(YES, field="inpatient", field_required="inpatient_days")
        self.m2m_required_if(YES, field="inpatient", m2m_field="inpatient_reasons")
        self.m2m_other_specify(
            m2m_field="inpatient_reasons", field_other="inpatient_reasons_other"
        )
        self.required_if(
            YES,
            field="inpatient",
            field_required="inpatient_cost",
            field_required_evaluate_as_int=True,
        )
        self.required_if(YES, field="inpatient", field_required="inpatient_accompany")
        self.required_if(YES, field="inpatient", field_required="inpatient_food")
        self.required_if(
            YES,
            field="inpatient_food",
            field_required="inpatient_food_cost",
            field_required_evaluate_as_int=True,
        )
        self.required_if(
            YES,
            field="inpatient",
            field_required="inpatient_nowork_days",
            field_required_evaluate_as_int=True,
        )
        self.required_if(YES, field="inpatient", field_required="inpatient_household_nowork")
        self.required_if(
            YES,
            field="inpatient_household_nowork",
            field_required="inpatient_household_nowork_days",
            field_required_evaluate_as_int=True,
        )

        self.m2m_other_specify(
            m2m_field="inpatient_money_sources", field_other="inpatient_money_sources_other"
        )

        if self.cleaned_data.get("inpatient_money_sources_main") and self.cleaned_data.get(
            "inpatient_money_sources_main"
        ) not in [o.name for o in self.cleaned_data.get("inpatient_money_sources")]:
            self.raise_validation_error(
                {
                    "inpatient_money_sources_main": _(
                        "Response not found among responses given above"
                    )
                },
                INVALID_ERROR,
            )
        self.validate_other_specify(
            field="inpatient_money_sources_main",
            other_specify_field="inpatient_money_sources_main_other",
        )


class CareseekingBForm(
    CrfSingletonModelFormMixin,
    CrfModelFormMixin,
    forms.ModelForm,
):
    form_validator_cls = CareseekingBFormValidator

    class Meta:
        model = CareseekingB
        fields = "__all__"



from django import forms
from edc_constants.constants import DM, HIV, HTN, NO, OTHER, YES
from edc_crf.modelform_mixins import CrfSingletonModelFormMixin
from edc_form_validators import INVALID_ERROR, FormValidator

from ...constants import ALONE, PAID_WORK
from ...models import CareseekingA
from ..mixins import CrfModelFormMixin


class CareseekingAFormValidator(FormValidator):
    def clean(self) -> None:
        self.m2m_other_specify(m2m_field="med_conditions", field_other="med_conditions_other")
        self.applicable_if(
            NO, field="med_collected", field_applicable="med_not_collected_reason"
        )
        self.validate_other_specify(
            field="med_collected", other_specify_field="med_not_collected_reason_other"
        )
        self.required_if_m2m(HIV, field="med_conditions", field_required="med_cost_hiv")
        self.required_if_m2m(HTN, field="med_conditions", field_required="med_cost_htn")
        self.required_if_m2m(DM, field="med_conditions", field_required="med_cost_dm")

        self.applicable_if(YES, field="tests_requested", field_applicable="tests_done")
        self.applicable_if(NO, field="tests_done", field_applicable="tests_not_done_reason")
        self.validate_other_specify(
            field="tests_not_done_reason", other_specify_field="tests_not_done_other"
        )
        self.required_if(YES, field="tests_done", field_required="tests_cost")

        self.validate_other_specify(
            field="missed_activities", other_specify_field="missed_activities_other"
        )
        self.required_if(
            PAID_WORK,
            OTHER,
            field="missed_activities",
            field_required="care_visit_lost_income",
            field_required_evaluate_as_int=True,
        )

        self.applicable_if(YES, field="referral", field_applicable="referral_type")
        self.applicable_if(YES, field="referral", field_applicable="referral_facility")

        self.m2m_single_selection_if(ALONE, m2m_field="accompany")
        if (
            self.cleaned_data.get("accompany_num")
            and ALONE in [o.name for o in self.cleaned_data.get("accompany")]
            and self.cleaned_data.get("accompany_num") != 0
        ):
            self.raise_validation_error(
                {"accompany_num": "Expected 0 based on response above"}, INVALID_ERROR
            )
        elif (
            self.cleaned_data.get("accompany_num")
            and ALONE not in [o.name for o in self.cleaned_data.get("accompany") or []]
            and self.cleaned_data.get("accompany_num") == 0
        ):
            self.raise_validation_error(
                {"accompany_num": "Expected a value greater than 0 based on response above"},
                INVALID_ERROR,
            )

        self.applicable_if_true(
            ALONE not in [o.name for o in self.cleaned_data.get("accompany") or []],
            field_applicable="accompany_wait",
        )
        self.applicable_if_true(
            ALONE not in [o.name for o in self.cleaned_data.get("accompany") or []],
            field_applicable="accompany_alt",
        )
        self.validate_other_specify(
            field="accompany_alt", other_specify_field="accompany_alt_other"
        )
        self.required_if(
            PAID_WORK,
            OTHER,
            field="accompany_alt",
            field_required="accompany_lost_income",
            field_required_evaluate_as_int=True,
        )

        self.m2m_other_specify(m2m_field="money_sources", field_other="money_sources_other")

        if self.cleaned_data.get("money_source_main") and self.cleaned_data.get(
            "money_source_main"
        ) not in [o.name for o in self.cleaned_data.get("money_sources")]:
            self.raise_validation_error(
                {"money_source_main": "Response not found among responses given above"},
                INVALID_ERROR,
            )
        self.validate_other_specify(
            field="money_source_main", other_specify_field="money_source_main_other"
        )


class CareseekingAForm(
    CrfSingletonModelFormMixin,
    CrfModelFormMixin,
    forms.ModelForm,
):
    form_validator_cls = CareseekingAFormValidator

    class Meta:
        model = CareseekingA
        fields = "__all__"
